#!/usr/bin/env python3
"""
System Hardening Tool
Automatically hardens system security settings and configurations.
Parameters:
    - mode: Hardening mode (basic, strict, custom)
    - config: Custom configuration file
Requires: admin/root privileges
"""
import subprocess
import logging
from datetime import datetime
import json
from pathlib import Path
import platform
import os
import sys
import re
import yaml
import shutil
import socket
if os.name != 'nt':
    import pwd
else:
    pwd = None
if os.name != 'nt':
    import grp
else:
    grp = None
import stat

class SystemHardener:
    def __init__(self, mode='basic', config_file=None):
        self.mode = mode
        self.config = self.load_config(config_file)
        self.is_windows = platform.system().lower() == 'windows'
        self.backup_dir = Path("security_backups")
        self.backup_dir.mkdir(exist_ok=True)
        self.setup_logging()
        
    def setup_logging(self):
        """Set up logging configuration"""
        log_dir = Path("hardening_logs")
        log_dir.mkdir(exist_ok=True)
        
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(log_dir / "hardening.log"),
                logging.StreamHandler()
            ]
        )
        
    def load_config(self, config_file):
        """Load hardening configuration"""
        default_config = {
            'file_permissions': {
                'sensitive_dirs': [
                    '/etc/passwd',
                    '/etc/shadow',
                    '/etc/group',
                    'C:\\Windows\\System32'
                ],
                'permissions': {
                    'files': '600',
                    'directories': '700'
                }
            },
            'network': {
                'disable_services': [
                    'telnet',
                    'ftp',
                    'rsh'
                ],
                'tcp_wrappers': {
                    'allow': ['127.0.0.1'],
                    'deny': ['ALL']
                }
            },
            'authentication': {
                'password_policy': {
                    'min_length': 12,
                    'complexity': True,
                    'history': 5,
                    'max_age': 90
                },
                'account_lockout': {
                    'threshold': 5,
                    'duration': 30
                }
            },
            'services': {
                'disable': [
                    'Guest',
                    'RemoteRegistry'
                ]
            },
            'registry': {
                'policies': {
                    r'SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System': {
                        'EnableLUA': 1,
                        'ConsentPromptBehaviorAdmin': 2,
                        'PromptOnSecureDesktop': 1
                    }
                }
            }
        }
        
        if config_file and Path(config_file).exists():
            try:
                with open(config_file, 'r') as f:
                    custom_config = yaml.safe_load(f)
                    # Merge custom config with defaults
                    for category in default_config:
                        if category in custom_config:
                            default_config[category].update(custom_config[category])
            except Exception as e:
                logging.error(f"Error loading config file: {str(e)}")
                
        return default_config
        
    def backup_file(self, filepath):
        """Create backup of file before modification"""
        try:
            if os.path.exists(filepath):
                backup_path = self.backup_dir / f"{Path(filepath).name}_{datetime.now().strftime('%Y%m%d_%H%M%S')}"
                shutil.copy2(filepath, backup_path)
                logging.info(f"Created backup: {backup_path}")
                return backup_path
        except Exception as e:
            logging.error(f"Error backing up {filepath}: {str(e)}")
        return None
        
    def harden_file_permissions(self):
        """Harden file and directory permissions"""
        logging.info("Hardening file permissions...")
        
        for dirpath in self.config['file_permissions']['sensitive_dirs']:
            if os.path.exists(dirpath):
                try:
                    # Backup before modification
                    self.backup_file(dirpath)
                    
                    if os.path.isfile(dirpath):
                        # Set file permissions
                        if self.is_windows:
                            subprocess.run([
                                'icacls', dirpath,
                                '/inheritance:d',  # Disable inheritance
                                '/grant:r', 'Administrators:(F)',  # Full control for admins
                                '/deny', 'Users:(W,D)'  # Deny write/delete for users
                            ], check=True)
                        else:
                            os.chmod(dirpath, int(self.config['file_permissions']['permissions']['files'], 8))
                            
                    else:
                        # Set directory permissions
                        if self.is_windows:
                            subprocess.run([
                                'icacls', dirpath,
                                '/inheritance:d',
                                '/grant:r', 'Administrators:(OI)(CI)(F)',
                                '/deny', 'Users:(OI)(CI)(W,D)'
                            ], check=True)
                        else:
                            os.chmod(dirpath, int(self.config['file_permissions']['permissions']['directories'], 8))
                            
                    logging.info(f"Hardened permissions for: {dirpath}")
                    
                except Exception as e:
                    logging.error(f"Error hardening permissions for {dirpath}: {str(e)}")
                    
    def harden_network(self):
        """Harden network configuration"""
        logging.info("Hardening network configuration...")
        
        try:
            if self.is_windows:
                # Disable network services using PowerShell
                for service in self.config['network']['disable_services']:
                    try:
                        subprocess.run([
                            'powershell',
                            f'Set-Service -Name {service} -StartupType Disabled'
                        ], check=True)
                        logging.info(f"Disabled service: {service}")
                    except:
                        continue
                        
                # Configure Windows Firewall
                subprocess.run([
                    'netsh', 'advfirewall', 'set', 'allprofiles',
                    'state', 'on'
                ], check=True)
                
                # Block unnecessary ports
                subprocess.run([
                    'netsh', 'advfirewall', 'firewall', 'add', 'rule',
                    'name="Block Telnet"',
                    'dir=in', 'action=block',
                    'protocol=TCP', 'localport=23'
                ], check=True)
                
            else:
                # Configure TCP Wrappers
                hosts_allow = Path("/etc/hosts.allow")
                hosts_deny = Path("/etc/hosts.deny")
                
                if hosts_allow.exists() and hosts_deny.exists():
                    # Backup files
                    self.backup_file(hosts_allow)
                    self.backup_file(hosts_deny)
                    
                    # Update hosts.allow
                    with open(hosts_allow, 'w') as f:
                        for ip in self.config['network']['tcp_wrappers']['allow']:
                            f.write(f"ALL: {ip}\n")
                            
                    # Update hosts.deny
                    with open(hosts_deny, 'w') as f:
                        f.write("ALL: ALL\n")
                        
                # Configure iptables
                subprocess.run(['iptables', '-F'], check=True)  # Flush rules
                subprocess.run(['iptables', '-P', 'INPUT', 'DROP'], check=True)
                subprocess.run(['iptables', '-P', 'FORWARD', 'DROP'], check=True)
                subprocess.run(['iptables', '-P', 'OUTPUT', 'ACCEPT'], check=True)
                
                # Allow established connections
                subprocess.run([
                    'iptables', '-A', 'INPUT', '-m', 'state',
                    '--state', 'ESTABLISHED,RELATED', '-j', 'ACCEPT'
                ], check=True)
                
                # Allow localhost
                subprocess.run([
                    'iptables', '-A', 'INPUT', '-i', 'lo', '-j', 'ACCEPT'
                ], check=True)
                
            logging.info("Network hardening completed")
            
        except Exception as e:
            logging.error(f"Error hardening network: {str(e)}")
            
    def harden_authentication(self):
        """Harden authentication settings"""
        logging.info("Hardening authentication settings...")
        
        try:
            if self.is_windows:
                # Configure password policy using PowerShell
                subprocess.run([
                    'powershell',
                    'Set-ExecutionPolicy Bypass -Scope Process -Force;',
                    '$securityPolicy = Get-Content $env:windir\\security\\templates\\securitytemplate.inf;',
                    f'$securityPolicy += "MinimumPasswordLength = {self.config["authentication"]["password_policy"]["min_length"]}";',
                    f'$securityPolicy += "PasswordComplexity = {1 if self.config["authentication"]["password_policy"]["complexity"] else 0}";',
                    f'$securityPolicy += "PasswordHistorySize = {self.config["authentication"]["password_policy"]["history"]}";',
                    f'$securityPolicy += "MaximumPasswordAge = {self.config["authentication"]["password_policy"]["max_age"]}";',
                    f'$securityPolicy += "LockoutBadCount = {self.config["authentication"]["account_lockout"]["threshold"]}";',
                    f'$securityPolicy += "LockoutDuration = {self.config["authentication"]["account_lockout"]["duration"]}";',
                    '$securityPolicy | Set-Content $env:windir\\security\\templates\\securitytemplate.inf;',
                    'secedit /configure /db secedit.sdb /cfg $env:windir\\security\\templates\\securitytemplate.inf'
                ], check=True)
                
            else:
                # Configure PAM settings
                login_defs = Path("/etc/login.defs")
                if login_defs.exists():
                    # Backup file
                    self.backup_file(login_defs)
                    
                    with open(login_defs, 'r') as f:
                        content = f.readlines()
                        
                    # Update password settings
                    settings = {
                        'PASS_MAX_DAYS': str(self.config['authentication']['password_policy']['max_age']),
                        'PASS_MIN_LEN': str(self.config['authentication']['password_policy']['min_length']),
                        'PASS_WARN_AGE': '7'
                    }
                    
                    for i, line in enumerate(content):
                        for setting, value in settings.items():
                            if line.startswith(setting):
                                content[i] = f"{setting}\t{value}\n"
                                
                    with open(login_defs, 'w') as f:
                        f.writelines(content)
                        
                # Configure PAM password quality
                pam_pwquality = Path("/etc/security/pwquality.conf")
                if pam_pwquality.exists():
                    self.backup_file(pam_pwquality)
                    with open(pam_pwquality, 'w') as f:
                        f.write("minlen = 12\n")
                        f.write("dcredit = -1\n")  # Require digit
                        f.write("ucredit = -1\n")  # Require uppercase
                        f.write("lcredit = -1\n")  # Require lowercase
                        f.write("ocredit = -1\n")  # Require special char
                        
            logging.info("Authentication hardening completed")
            
        except Exception as e:
            logging.error(f"Error hardening authentication: {str(e)}")
            
    def harden_services(self):
        """Harden system services"""
        logging.info("Hardening system services...")
        
        try:
            if self.is_windows:
                # Disable unnecessary services
                for service in self.config['services']['disable']:
                    try:
                        subprocess.run([
                            'sc', 'config', service,
                            'start=', 'disabled'
                        ], check=True)
                        subprocess.run([
                            'sc', 'stop', service
                        ], check=True)
                        logging.info(f"Disabled service: {service}")
                    except:
                        continue
                        
                # Configure service permissions
                subprocess.run([
                    'sc', 'sdset', 'scmanager',
                    'D:(A;;CC;;;AU)(A;;CCLCRPRC;;;IU)(A;;CCLCRPRC;;;SU)(A;;CCLCRPWPRC;;;SY)(A;;KA;;;BA)S:(AU;FA;KA;;;WD)(AU;OIIOFA;GA;;;WD)'
                ], check=True)
                
            else:
                # Disable unnecessary services using systemctl
                services = [
                    'telnet',
                    'rsh',
                    'rlogin',
                    'rexec',
                    'autofs'
                ]
                
                for service in services:
                    try:
                        subprocess.run([
                            'systemctl', 'disable', service
                        ], check=True)
                        subprocess.run([
                            'systemctl', 'stop', service
                        ], check=True)
                        logging.info(f"Disabled service: {service}")
                    except:
                        continue
                        
            logging.info("Service hardening completed")
            
        except Exception as e:
            logging.error(f"Error hardening services: {str(e)}")
            
    def harden_registry(self):
        """Harden Windows Registry settings"""
        if not self.is_windows:
            return
            
        logging.info("Hardening Windows Registry...")
        
        try:
            import winreg
        except ImportError:
            logging.error("winreg module not found")
            return
        
        try:
            for key_path, values in self.config['registry']['policies'].items():
                try:
                    # Open or create key
                    key = winreg.CreateKeyEx(
                        winreg.HKEY_LOCAL_MACHINE,
                        key_path,
                        0,
                        winreg.KEY_WRITE
                    )
                    
                    # Set values
                    for name, data in values.items():
                        winreg.SetValueEx(
                            key,
                            name,
                            0,
                            winreg.REG_DWORD,
                            data
                        )
                        logging.info(f"Set registry value: {key_path}\\{name} = {data}")
                        
                    winreg.CloseKey(key)
                    
                except Exception as e:
                    logging.error(f"Error setting registry key {key_path}: {str(e)}")
                    
            logging.info("Registry hardening completed")
            
        except Exception as e:
            logging.error(f"Error hardening registry: {str(e)}")
            
    def apply_hardening(self):
        """Apply system hardening measures"""
        logging.info(f"Starting system hardening - Mode: {self.mode}")
        
        # Check privileges
        if os.geteuid() != 0:
            logging.error("This script must be run as administrator/root")
            sys.exit(1)
            
        try:
            # Apply hardening measures based on mode
            if self.mode in ['basic', 'strict']:
                self.harden_file_permissions()
                self.harden_network()
                self.harden_authentication()
                self.harden_services()
                
            if self.mode == 'strict':
                # Additional strict mode hardening
                if self.is_windows:
                    self.harden_registry()
                    
            # Save final state
            self.save_state()
            
        except Exception as e:
            logging.error(f"Error during hardening: {str(e)}")
            
    def save_state(self):
        """Save hardening state and results"""
        state = {
            'timestamp': datetime.now().isoformat(),
            'mode': self.mode,
            'system': platform.system(),
            'backups': [str(p) for p in self.backup_dir.glob('*')],
            'config': self.config
        }
        
        # Save to file
        output_dir = Path("hardening_logs")
        output_file = output_dir / f"state_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(output_file, 'w') as f:
            json.dump(state, f, indent=2)
            
        logging.info(f"State saved to {output_file}")
        
        # Print summary
        print("\nSystem Hardening Summary")
        print("=" * 50)
        print(f"Mode: {self.mode}")
        print(f"System: {platform.system()}")
        print(f"Backup Location: {self.backup_dir}")
        print("\nHardening Measures Applied:")
        print("- File Permissions")
        print("- Network Security")
        print("- Authentication Settings")
        print("- Service Hardening")
        if self.is_windows and self.mode == 'strict':
            print("- Registry Hardening")
            
        print("\nBackup files created:", len(state['backups']))
        print("\nReview the logs for detailed information and any errors.")

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="System Hardening Tool")
    parser.add_argument("--mode", choices=['basic', 'strict', 'custom'],
                      default='basic', help="Hardening mode")
    parser.add_argument("--config", help="Custom configuration file")
    args = parser.parse_args()
    
    hardener = SystemHardener(args.mode, args.config)
    hardener.apply_hardening()
